@@ -22,16 +22,16 @@ def profile_group():
 
 
 @profile_group.command('list', cls=Command)
-def profile_list(lockdown: LockdownClient):
+def profile_list(service_provider: LockdownClient):
     """ list installed profiles """
-    print_json(MobileConfigService(lockdown=lockdown).get_profile_list())
+    print_json(MobileConfigService(lockdown=service_provider).get_profile_list())
 
 
 @profile_group.command('install', cls=Command)
 @click.argument('profiles', nargs=-1, type=click.File('rb'))
-def profile_install(lockdown: LockdownClient, profiles):
+def profile_install(service_provider: LockdownClient, profiles):
     """ install given profiles """
-    service = MobileConfigService(lockdown=lockdown)
+    service = MobileConfigService(lockdown=service_provider)
     for profile in profiles:
         logger.info(f'installing {profile.name}')
         service.install_profile(profile.read())
@@ -43,9 +43,9 @@ def profile_install(lockdown: LockdownClient, profiles):
 @click.option('--keystore-password', prompt=True, required=True, hide_input=True,
               help="The password for the PKCS#12 keystore.")
 @click.argument('profiles', nargs=-1, type=click.File('rb'))
-def profile_install_silent(lockdown: LockdownClient, profiles, keystore, keystore_password):
+def profile_install_silent(service_provider: LockdownClient, profiles, keystore, keystore_password):
     """ install given profiles without user interaction (requires the device to be supervised) """
-    service = MobileConfigService(lockdown=lockdown)
+    service = MobileConfigService(lockdown=service_provider)
     for profile in profiles:
         logger.info(f'installing {profile.name}')
         service.install_profile_silent(
@@ -53,17 +53,16 @@ def profile_install_silent(lockdown: LockdownClient, profiles, keystore, keystor
 
 
 @profile_group.command('cloud-configuration', cls=Command)
-@click.option('--color/--no-color', default=True)
-def profile_cloud_configuration(lockdown: LockdownClient, color):
+def profile_cloud_configuration(service_provider: LockdownClient):
     """ get cloud configuration """
-    print_json(MobileConfigService(lockdown=lockdown).get_cloud_configuration(), colored=color)
+    print_json(MobileConfigService(lockdown=service_provider).get_cloud_configuration())
 
 
 @profile_group.command('store', cls=Command)
 @click.argument('profiles', nargs=-1, type=click.File('rb'))
-def profile_store(lockdown: LockdownClient, profiles):
+def profile_store(service_provider: LockdownClient, profiles):
     """ store profile """
-    service = MobileConfigService(lockdown=lockdown)
+    service = MobileConfigService(lockdown=service_provider)
     for profile in profiles:
         logger.info(f'storing {profile.name}')
         service.store_profile(profile.read())
@@ -71,13 +70,13 @@ def profile_store(lockdown: LockdownClient, profiles):
 
 @profile_group.command('remove', cls=Command)
 @click.argument('name')
-def profile_remove(lockdown: LockdownClient, name):
+def profile_remove(service_provider: LockdownClient, name):
     """ remove profile by name """
-    MobileConfigService(lockdown=lockdown).remove_profile(name)
+    MobileConfigService(lockdown=service_provider).remove_profile(name)
 
 
 @profile_group.command('set-wifi-power', cls=Command)
 @click.argument('state', type=click.Choice(['on', 'off']), required=False)
-def profile_set_wifi_power(lockdown: LockdownClient, state):
+def profile_set_wifi_power(service_provider: LockdownClient, state):
     """ change Wi-Fi power state """
-    MobileConfigService(lockdown=lockdown).set_wifi_power_state(state == 'on')
+    MobileConfigService(lockdown=service_provider).set_wifi_power_state(state == 'on')
