@@ -10,21 +10,27 @@ from contextlib import contextmanager, suppress
 from enum import Enum
 from functools import wraps
 from pathlib import Path
+from ssl import SSLZeroReturnError
 from typing import Dict, Mapping, Optional
 
+from cryptography.hazmat.primitives import hashes
+from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey
+from cryptography.hazmat.primitives.serialization import Encoding
+from cryptography.hazmat.primitives.serialization.pkcs7 import PKCS7SignatureBuilder
+from cryptography.hazmat.primitives.serialization.pkcs12 import load_pkcs12
 from packaging.version import Version
 
 from pymobiledevice3 import usbmux
 from pymobiledevice3.ca import ca_do_everything
 from pymobiledevice3.exceptions import CannotStopSessionError, ConnectionTerminatedError, FatalPairingError, \
-    IncorrectModeError, InvalidConnectionError, InvalidHostIDError, InvalidServiceError, LockdownError, \
-    MissingValueError, NotPairedError, PairingDialogResponsePendingError, PairingError, PasswordRequiredError, \
-    SetProhibitedError, StartServiceError, UserDeniedPairingError
+    GetProhibitedError, IncorrectModeError, InvalidConnectionError, InvalidHostIDError, InvalidServiceError, \
+    LockdownError, MissingValueError, NotPairedError, PairingDialogResponsePendingError, PairingError, \
+    PasswordRequiredError, SetProhibitedError, StartServiceError, UserDeniedPairingError
 from pymobiledevice3.irecv_devices import IRECV_DEVICES
 from pymobiledevice3.lockdown_service_provider import LockdownServiceProvider
 from pymobiledevice3.pair_records import create_pairing_records_cache_folder, generate_host_id, \
     get_preferred_pair_record
-from pymobiledevice3.service_connection import LockdownServiceConnection
+from pymobiledevice3.service_connection import ServiceConnection
 from pymobiledevice3.usbmux import PlistMuxConnection
 
 SYSTEM_BUID = '30142955-444094379208051516'
@@ -55,12 +61,7 @@ def _reconnect_on_remote_close(f):
         except (BrokenPipeError, ConnectionTerminatedError):
             self: LockdownClient = args[0]
 
-            # first we release the socket on our end to avoid a ResourceWarning
-            self.close()
-
-            # now we re-establish the connection
-            self.logger.debug('remote device closed the connection. reconnecting...')
-            self.service = self._create_service_connection(self.port)
+            self._reestablish_connection()
             self.validate_pairing()
             return f(*args, **kwargs)
 
@@ -68,7 +69,7 @@ def _reconnect_on_remote_close(f):
 
 
 class LockdownClient(ABC, LockdownServiceProvider):
-    def __init__(self, service: LockdownServiceConnection, host_id: str, identifier: str = None,
+    def __init__(self, service: ServiceConnection, host_id: str, identifier: str = None,
                  label: str = DEFAULT_LABEL, system_buid: str = SYSTEM_BUID, pair_record: Mapping = None,
                  pairing_records_cache_folder: Path = None, port: int = SERVICE_PORT):
         """
@@ -106,10 +107,10 @@ class LockdownClient(ABC, LockdownServiceProvider):
         self.product_type = self.all_values.get('ProductType')
 
     @classmethod
-    def create(cls, service: LockdownServiceConnection, identifier: str = None, system_buid: str = SYSTEM_BUID,
-               label: str = DEFAULT_LABEL, autopair: bool = True, pair_timeout: int = None, local_hostname: str = None,
+    def create(cls, service: ServiceConnection, identifier: str = None, system_buid: str = SYSTEM_BUID,
+               label: str = DEFAULT_LABEL, autopair: bool = True, pair_timeout: float = None, local_hostname: str = None,
                pair_record: Mapping = None, pairing_records_cache_folder: Path = None, port: int = SERVICE_PORT,
-               **cls_specific_args):
+               private_key: Optional[RSAPrivateKey] = None, **cls_specific_args):
         """
         Create a LockdownClient instance
 
@@ -123,6 +124,7 @@ class LockdownClient(ABC, LockdownServiceProvider):
         :param pair_record: Use this pair record instead of the default behavior (search in host/create our own)
         :param pairing_records_cache_folder: Use the following location to search and save pair records
         :param port: lockdownd service port
+        :param private_key: Used to pass custom RSA key for pairing purposes, if None it will be autogenerated
         :param cls_specific_args: Additional members to pass into LockdownClient subclasses
         :return: LockdownClient subclass
         """
@@ -133,8 +135,7 @@ class LockdownClient(ABC, LockdownServiceProvider):
             service, host_id=host_id, identifier=identifier, label=label, system_buid=system_buid,
             pair_record=pair_record, pairing_records_cache_folder=pairing_records_cache_folder, port=port,
             **cls_specific_args)
-
-        lockdown_client._handle_autopair(autopair, pair_timeout)
+        lockdown_client._handle_autopair(autopair, pair_timeout, private_key=private_key)
         return lockdown_client
 
     def __repr__(self) -> str:
@@ -310,7 +311,12 @@ class LockdownClient(ABC, LockdownServiceProvider):
         self.session_id = start_session.get('SessionID')
         if start_session.get('EnableSessionSSL'):
             with self.ssl_file() as f:
-                self.service.ssl_start(f)
+                try:
+                    self.service.ssl_start(f)
+                except SSLZeroReturnError:
+                    # possible when we have a pair record, but it was removed on-device
+                    self._reestablish_connection()
+                    return False
 
         self.paired = True
 
@@ -321,7 +327,7 @@ class LockdownClient(ABC, LockdownServiceProvider):
         return True
 
     @_reconnect_on_remote_close
-    def pair(self, timeout: int = None) -> None:
+    def pair(self, timeout: float = None, private_key: Optional[RSAPrivateKey] = None) -> None:
         self.device_public_key = self.get_value('', 'DevicePublicKey')
         if not self.device_public_key:
             self.logger.error('Unable to retrieve DevicePublicKey')
@@ -329,7 +335,7 @@ class LockdownClient(ABC, LockdownServiceProvider):
             raise PairingError()
 
         self.logger.info('Creating host key & certificate')
-        cert_pem, private_key_pem, device_certificate = ca_do_everything(self.device_public_key)
+        cert_pem, private_key_pem, device_certificate = ca_do_everything(self.device_public_key, private_key=private_key)
 
         pair_record = {'DevicePublicKey': self.device_public_key,
                        'DeviceCertificate': device_certificate,
@@ -355,6 +361,63 @@ class LockdownClient(ABC, LockdownServiceProvider):
         self.save_pair_record()
         self.paired = True
 
+    @_reconnect_on_remote_close
+    def pair_supervised(self, timeout: float = None, p12file: Path = None, password: str = None) -> None:
+
+        keystore_data = p12file.read()
+        try:
+            decrypted_p12 = load_pkcs12(
+                keystore_data, password.encode('utf-8'))
+        except Exception as pkcs12_error:
+            self.service.close()
+            raise Exception(f'load_pkcs12 error: {pkcs12_error}')
+
+        self.device_public_key = self.get_value('', 'DevicePublicKey')
+        if not self.device_public_key:
+            self.logger.error('Unable to retrieve DevicePublicKey')
+            self.service.close()
+            raise PairingError()
+
+        self.logger.info('Creating host key & certificate')
+        cert_pem, private_key_pem, device_certificate = ca_do_everything(
+            self.device_public_key)
+
+        pair_record = {'DevicePublicKey': self.device_public_key,
+                       'DeviceCertificate': device_certificate,
+                       'HostCertificate': cert_pem,
+                       'HostID': self.host_id,
+                       'RootCertificate': cert_pem,
+                       'RootPrivateKey': private_key_pem,
+                       'WiFiMACAddress': self.wifi_mac_address,
+                       'SystemBUID': self.system_buid}
+
+        pair_options = {'PairRecord': pair_record, 'ProtocolVersion': '2',
+                        'PairingOptions': {'SupervisorCertificate': decrypted_p12.cert.certificate.public_bytes(Encoding.DER),
+                                           'ExtendedPairingErrors': True}}
+
+        # first pair with SupervisorCertificate as PairingOptions to get Challenge
+        pair = self._request_pair(pair_options, timeout=timeout)
+        if pair.get('Error') == 'MCChallengeRequired':
+            extendedresponse = pair.get('ExtendedResponse')
+            if extendedresponse is not None:
+                pairingchallenge = extendedresponse.get('PairingChallenge')
+                signed_response = PKCS7SignatureBuilder().set_data(pairingchallenge).add_signer(
+                    decrypted_p12.cert.certificate, decrypted_p12.key, hashes.SHA256()).sign(Encoding.DER, [])
+                pair_options = {'PairRecord': pair_record, 'ProtocolVersion': '2', 'PairingOptions': {
+                    'ChallengeResponse': signed_response, 'ExtendedPairingErrors': True}}
+                # second pair with Response to Challenge
+                pair = self._request_pair(pair_options, timeout=timeout)
+
+        pair_record['HostPrivateKey'] = private_key_pem
+        escrow_bag = pair.get('EscrowBag')
+
+        if escrow_bag is not None:
+            pair_record['EscrowBag'] = pair.get('EscrowBag')
+
+        self.pair_record = pair_record
+        self.save_pair_record()
+        self.paired = True
+
     @_reconnect_on_remote_close
     def unpair(self, host_id: str = None) -> None:
         pair_record = self.pair_record if host_id is None else {'HostID': host_id}
@@ -420,7 +483,7 @@ class LockdownClient(ABC, LockdownServiceProvider):
         return response
 
     @_reconnect_on_remote_close
-    def start_lockdown_service(self, name: str, include_escrow_bag: bool = False) -> LockdownServiceConnection:
+    def start_lockdown_service(self, name: str, include_escrow_bag: bool = False) -> ServiceConnection:
         attr = self.get_service_connection_attributes(name, include_escrow_bag=include_escrow_bag)
         service_connection = self._create_service_connection(attr['Port'])
 
@@ -430,7 +493,7 @@ class LockdownClient(ABC, LockdownServiceProvider):
         return service_connection
 
     async def aio_start_lockdown_service(
-            self, name: str, include_escrow_bag: bool = False) -> LockdownServiceConnection:
+            self, name: str, include_escrow_bag: bool = False) -> ServiceConnection:
         attr = self.get_service_connection_attributes(name, include_escrow_bag=include_escrow_bag)
         service_connection = self._create_service_connection(attr['Port'])
 
@@ -458,7 +521,7 @@ class LockdownClient(ABC, LockdownServiceProvider):
         finally:
             os.unlink(filename)
 
-    def _handle_autopair(self, autopair: bool, timeout: int) -> None:
+    def _handle_autopair(self, autopair: bool, timeout: float, private_key: Optional[RSAPrivateKey] = None) -> None:
         if self.validate_pairing():
             return
 
@@ -466,13 +529,13 @@ class LockdownClient(ABC, LockdownServiceProvider):
         if not autopair:
             # but pairing by default was not requested
             return
-        self.pair(timeout=timeout)
+        self.pair(timeout=timeout, private_key=private_key)
         # get session_id
         if not self.validate_pairing():
             raise FatalPairingError()
 
     @abstractmethod
-    def _create_service_connection(self, port: int) -> LockdownServiceConnection:
+    def _create_service_connection(self, port: int) -> ServiceConnection:
         """ Used to establish a new ServiceConnection to a given port """
         pass
 
@@ -487,10 +550,14 @@ class LockdownClient(ABC, LockdownServiceProvider):
 
         error = response.get('Error')
         if error is not None:
+            # return response if supervisor cert challenge is required, to work with pair_supervisor
+            if error == 'MCChallengeRequired':
+                return response
             exception_errors = {'PasswordProtected': PasswordRequiredError,
                                 'PairingDialogResponsePending': PairingDialogResponsePendingError,
                                 'UserDeniedPairing': UserDeniedPairingError,
                                 'InvalidHostID': InvalidHostIDError,
+                                'GetProhibited': GetProhibitedError,
                                 'SetProhibited': SetProhibitedError,
                                 'MissingValue': MissingValueError,
                                 'InvalidService': InvalidServiceError,
@@ -503,7 +570,7 @@ class LockdownClient(ABC, LockdownServiceProvider):
 
         return response
 
-    def _request_pair(self, pair_options: Mapping, timeout: int = None) -> Mapping:
+    def _request_pair(self, pair_options: Mapping, timeout: float = None) -> Mapping:
         try:
             return self._request('Pair', pair_options)
         except PairingDialogResponsePendingError:
@@ -526,9 +593,13 @@ class LockdownClient(ABC, LockdownServiceProvider):
         pair_record_file = self.pairing_records_cache_folder / f'{self.identifier}.plist'
         pair_record_file.write_bytes(plistlib.dumps(self.pair_record))
 
+    def _reestablish_connection(self) -> None:
+        self.close()
+        self.service = self._create_service_connection(self.port)
+
 
 class UsbmuxLockdownClient(LockdownClient):
-    def __init__(self, service: LockdownServiceConnection, host_id: str, identifier: str = None,
+    def __init__(self, service: ServiceConnection, host_id: str, identifier: str = None,
                  label: str = DEFAULT_LABEL, system_buid: str = SYSTEM_BUID, pair_record: Mapping = None,
                  pairing_records_cache_folder: Path = None, port: int = SERVICE_PORT,
                  usbmux_address: Optional[str] = None):
@@ -547,10 +618,10 @@ class UsbmuxLockdownClient(LockdownClient):
             self.pair_record = get_preferred_pair_record(self.identifier, self.pairing_records_cache_folder,
                                                          usbmux_address=self.usbmux_address)
 
-    def _create_service_connection(self, port: int) -> LockdownServiceConnection:
-        return LockdownServiceConnection.create_using_usbmux(self.identifier, port,
-                                                             self.service.mux_device.connection_type,
-                                                             usbmux_address=self.usbmux_address)
+    def _create_service_connection(self, port: int) -> ServiceConnection:
+        return ServiceConnection.create_using_usbmux(self.identifier, port,
+                                                     self.service.mux_device.connection_type,
+                                                     usbmux_address=self.usbmux_address)
 
 
 class PlistUsbmuxLockdownClient(UsbmuxLockdownClient):
@@ -562,7 +633,7 @@ class PlistUsbmuxLockdownClient(UsbmuxLockdownClient):
 
 
 class TcpLockdownClient(LockdownClient):
-    def __init__(self, service: LockdownServiceConnection, host_id: str, hostname: str, identifier: str = None,
+    def __init__(self, service: ServiceConnection, host_id: str, hostname: str, identifier: str = None,
                  label: str = DEFAULT_LABEL, system_buid: str = SYSTEM_BUID, pair_record: Mapping = None,
                  pairing_records_cache_folder: Path = None, port: int = SERVICE_PORT, keep_alive: bool = True):
         """
@@ -584,26 +655,26 @@ class TcpLockdownClient(LockdownClient):
         self._keep_alive = keep_alive
         self.hostname = hostname
 
-    def _create_service_connection(self, port: int) -> LockdownServiceConnection:
-        return LockdownServiceConnection.create_using_tcp(self.hostname, port, keep_alive=self._keep_alive)
+    def _create_service_connection(self, port: int) -> ServiceConnection:
+        return ServiceConnection.create_using_tcp(self.hostname, port, keep_alive=self._keep_alive)
 
 
 class RemoteLockdownClient(LockdownClient):
-    def _create_service_connection(self, port: int) -> LockdownServiceConnection:
+    def _create_service_connection(self, port: int) -> ServiceConnection:
         raise NotImplementedError(
             'RemoteXPC service connections should only be created using RemoteServiceDiscoveryService')
 
-    def _handle_autopair(self, *args):
+    def _handle_autopair(self, *args, **kwargs):
         # The RemoteXPC version of lockdown doesn't support pairing operations
         return None
 
-    def pair(self, timeout: int = None) -> None:
+    def pair(self, *args, **kwargs) -> None:
         raise NotImplementedError('RemoteXPC lockdown version does not support pairing operations')
 
-    def unpair(self, timeout: int = None) -> None:
+    def unpair(self, timeout: float = None) -> None:
         raise NotImplementedError('RemoteXPC lockdown version does not support pairing operations')
 
-    def __init__(self, service: LockdownServiceConnection, host_id: str, identifier: str = None,
+    def __init__(self, service: ServiceConnection, host_id: str, identifier: str = None,
                  label: str = DEFAULT_LABEL, system_buid: str = SYSTEM_BUID, pair_record: Mapping = None,
                  pairing_records_cache_folder: Path = None, port: int = SERVICE_PORT):
         """
@@ -623,7 +694,7 @@ class RemoteLockdownClient(LockdownClient):
 
 
 def create_using_usbmux(serial: str = None, identifier: str = None, label: str = DEFAULT_LABEL, autopair: bool = True,
-                        connection_type: str = None, pair_timeout: int = None, local_hostname: str = None,
+                        connection_type: str = None, pair_timeout: float = None, local_hostname: str = None,
                         pair_record: Mapping = None, pairing_records_cache_folder: Path = None,
                         port: int = SERVICE_PORT, usbmux_address: Optional[str] = None) -> UsbmuxLockdownClient:
     """
@@ -642,8 +713,8 @@ def create_using_usbmux(serial: str = None, identifier: str = None, label: str =
     :param usbmux_address: usbmuxd address
     :return: UsbmuxLockdownClient instance
     """
-    service = LockdownServiceConnection.create_using_usbmux(serial, port, connection_type=connection_type,
-                                                            usbmux_address=usbmux_address)
+    service = ServiceConnection.create_using_usbmux(serial, port, connection_type=connection_type,
+                                                    usbmux_address=usbmux_address)
     cls = UsbmuxLockdownClient
     with usbmux.create_mux(usbmux_address=usbmux_address) as client:
         if isinstance(client, PlistMuxConnection):
@@ -662,7 +733,7 @@ def create_using_usbmux(serial: str = None, identifier: str = None, label: str =
 
 
 def create_using_tcp(hostname: str, identifier: str = None, label: str = DEFAULT_LABEL, autopair: bool = True,
-                     pair_timeout: int = None, local_hostname: str = None, pair_record: Mapping = None,
+                     pair_timeout: float = None, local_hostname: str = None, pair_record: Mapping = None,
                      pairing_records_cache_folder: Path = None, port: int = SERVICE_PORT,
                      keep_alive: bool = False) -> TcpLockdownClient:
     """
@@ -680,7 +751,7 @@ def create_using_tcp(hostname: str, identifier: str = None, label: str = DEFAULT
     :param keep_alive: use keep-alive to get notified when the connection is lost
     :return: TcpLockdownClient instance
     """
-    service = LockdownServiceConnection.create_using_tcp(hostname, port, keep_alive=keep_alive)
+    service = ServiceConnection.create_using_tcp(hostname, port, keep_alive=keep_alive)
     client = TcpLockdownClient.create(
         service, identifier=identifier, label=label, local_hostname=local_hostname, pair_record=pair_record,
         pairing_records_cache_folder=pairing_records_cache_folder, pair_timeout=pair_timeout, autopair=autopair,
@@ -688,14 +759,14 @@ def create_using_tcp(hostname: str, identifier: str = None, label: str = DEFAULT
     return client
 
 
-def create_using_remote(service: LockdownServiceConnection, identifier: str = None, label: str = DEFAULT_LABEL,
-                        autopair: bool = True, pair_timeout: int = None, local_hostname: str = None,
+def create_using_remote(service: ServiceConnection, identifier: str = None, label: str = DEFAULT_LABEL,
+                        autopair: bool = True, pair_timeout: float = None, local_hostname: str = None,
                         pair_record: Mapping = None, pairing_records_cache_folder: Path = None,
                         port: int = SERVICE_PORT) -> RemoteLockdownClient:
     """
     Create a TcpLockdownClient instance over RSD
 
-    :param hostname: The target device hostname
+    :param service: service connection to use
     :param identifier: Used as an identifier to look for the device pair record
     :param label: lockdownd user-agent
     :param autopair: Attempt to pair with device (blocking) if not already paired
