@@ -1,26 +1,19 @@
 import logging
-import os
 import platform
 import plistlib
-import sys
 import uuid
 from contextlib import suppress
 from pathlib import Path
-from typing import Mapping, Optional
+from typing import Generator, Mapping, Optional
 
 from pymobiledevice3 import usbmux
 from pymobiledevice3.common import get_home_folder
 from pymobiledevice3.exceptions import MuxException, NotPairedError
+from pymobiledevice3.osu.os_utils import get_os_utils
 from pymobiledevice3.usbmux import PlistMuxConnection
 
-PAIR_RECORDS_PATH = {
-    'win32': Path(os.environ.get('ALLUSERSPROFILE', ''), 'Apple', 'Lockdown'),
-    'darwin': Path('/var/db/lockdown/'),
-    'linux': Path('/var/lib/lockdown/'),
-}
-
 logger = logging.getLogger(__name__)
-
+OSUTILS = get_os_utils()
 PAIRING_RECORD_EXT = 'plist'
 
 
@@ -31,8 +24,7 @@ def generate_host_id(hostname: str = None) -> str:
 
 
 def get_itunes_pairing_record(identifier: str) -> Optional[Mapping]:
-    platform_type = 'linux' if not sys.platform.startswith('linux') else sys.platform
-    filename = PAIR_RECORDS_PATH[platform_type] / f'{identifier}.plist'
+    filename = OSUTILS.pair_record_path / f'{identifier}.plist'
     try:
         with open(filename, 'rb') as f:
             pair_record = plistlib.load(f)
@@ -81,8 +73,18 @@ def create_pairing_records_cache_folder(pairing_records_cache_folder: Path = Non
         pairing_records_cache_folder = get_home_folder()
     else:
         pairing_records_cache_folder.mkdir(parents=True, exist_ok=True)
+    OSUTILS.chown_to_non_sudo_if_needed(pairing_records_cache_folder)
     return pairing_records_cache_folder
 
 
 def get_remote_pairing_record_filename(identifier: str) -> str:
     return f'remote_{identifier}'
+
+
+def iter_remote_pair_records() -> Generator[Path, None, None]:
+    return get_home_folder().glob('remote_*')
+
+
+def iter_remote_paired_identifiers() -> Generator[str, None, None]:
+    for file in iter_remote_pair_records():
+        yield file.parts[-1].split('remote_', 1)[1].split('.', 1)[0]
