@@ -14,22 +14,28 @@ class DeviceInfoService(CoreDeviceService):
     def __init__(self, rsd: RemoteServiceDiscoveryService):
         super().__init__(rsd, self.SERVICE_NAME)
 
-    def get_device_info(self) -> Mapping:
+    async def get_device_info(self) -> Mapping:
         """
         Get device information
         """
-        return self.invoke('com.apple.coredevice.feature.getdeviceinfo', {})
+        return await self.invoke('com.apple.coredevice.feature.getdeviceinfo', {})
 
-    def query_mobilegestalt(self, keys: List[str]) -> Mapping:
+    async def get_display_info(self) -> Mapping:
+        """
+        Get display information
+        """
+        return await self.invoke('com.apple.coredevice.feature.getdisplayinfo', {})
+
+    async def query_mobilegestalt(self, keys: List[str]) -> Mapping:
         """
         Query MobileGestalt.
 
         Can only be performed to specific devices
         """
-        return self.invoke('com.apple.coredevice.feature.querymobilegestalt', {'keys': keys})
+        return await self.invoke('com.apple.coredevice.feature.querymobilegestalt', {'keys': keys})
 
-    def get_lockstate(self) -> Mapping:
+    async def get_lockstate(self) -> Mapping:
         """
         Get lockstate
         """
-        return self.invoke('com.apple.coredevice.feature.getlockstate', {})
+        return await self.invoke('com.apple.coredevice.feature.getlockstate', {})
