@@ -5,6 +5,7 @@ import socket
 import ssl
 import struct
 import time
+from enum import Enum
 from typing import Mapping, Optional
 
 import IPython
@@ -12,13 +13,13 @@ from pygments import formatters, highlight, lexers
 
 from pymobiledevice3.exceptions import ConnectionFailedError, ConnectionTerminatedError, NoDeviceConnectedError, \
     PyMobileDevice3Exception
+from pymobiledevice3.osu.os_utils import get_os_utils
 from pymobiledevice3.usbmux import MuxDevice, select_device
-from pymobiledevice3.utils import set_keepalive
 
 DEFAULT_AFTER_IDLE_SEC = 3
 DEFAULT_INTERVAL_SEC = 3
 DEFAULT_MAX_FAILS = 3
-
+OSUTIL = get_os_utils()
 SHELL_USAGE = """
 # This shell allows you to communicate directly with every service layer behind the lockdownd daemon.
 
@@ -36,7 +37,7 @@ print(client.recvall(20))
 """
 
 
-def build_plist(d, endianity='>', fmt=plistlib.FMT_XML):
+def build_plist(d: Mapping, endianity: str = '>', fmt: Enum = plistlib.FMT_XML) -> bytes:
     payload = plistlib.dumps(d, fmt=fmt)
     message = struct.pack(endianity + 'L', len(payload))
     return message + payload
@@ -61,36 +62,37 @@ def create_context(certfile, keyfile=None):
     return context
 
 
-class LockdownServiceConnection:
-    """ wrapper for usbmux tcp-relay connections """
+class ServiceConnection:
+    """ wrapper for tcp-relay connections """
 
     def __init__(self, sock: socket.socket, mux_device: MuxDevice = None):
         self.logger = logging.getLogger(__name__)
         self.socket = sock
+        self._offset = 0
 
         # usbmux connections contain additional information associated with the current connection
         self.mux_device = mux_device
 
-        self._reader = None  # type: Optional[asyncio.StreamReader]
-        self._writer = None  # type: Optional[asyncio.StreamWriter]
+        self.reader = None  # type: Optional[asyncio.StreamReader]
+        self.writer = None  # type: Optional[asyncio.StreamWriter]
 
     @staticmethod
-    def create_using_tcp(hostname: str, port: int, keep_alive: bool = True) -> 'LockdownServiceConnection':
+    def create_using_tcp(hostname: str, port: int, keep_alive: bool = True) -> 'ServiceConnection':
         sock = socket.create_connection((hostname, port))
         if keep_alive:
-            set_keepalive(sock)
-        return LockdownServiceConnection(sock)
+            OSUTIL.set_keepalive(sock)
+        return ServiceConnection(sock)
 
     @staticmethod
     def create_using_usbmux(udid: Optional[str], port: int, connection_type: str = None,
-                            usbmux_address: Optional[str] = None) -> 'LockdownServiceConnection':
+                            usbmux_address: Optional[str] = None) -> 'ServiceConnection':
         target_device = select_device(udid, connection_type=connection_type, usbmux_address=usbmux_address)
         if target_device is None:
             if udid:
                 raise ConnectionFailedError()
             raise NoDeviceConnectedError()
         sock = target_device.connect(port, usbmux_address=usbmux_address)
-        return LockdownServiceConnection(sock, mux_device=target_device)
+        return ServiceConnection(sock, mux_device=target_device)
 
     def setblocking(self, blocking: bool) -> None:
         self.socket.setblocking(blocking)
@@ -99,15 +101,15 @@ class LockdownServiceConnection:
         self.socket.close()
 
     async def aio_close(self) -> None:
-        if self._writer is None:
+        if self.writer is None:
             return
-        self._writer.close()
+        self.writer.close()
         try:
-            await self._writer.wait_closed()
+            await self.writer.wait_closed()
         except ssl.SSLError:
             pass
-        self._writer = None
-        self._reader = None
+        self.writer = None
+        self.reader = None
 
     def recv(self, length=4096) -> bytes:
         """ socket.recv() normal behavior. attempt to receive a single chunk """
@@ -145,11 +147,15 @@ class LockdownServiceConnection:
                 # Allow ssl to do stuff
                 time.sleep(0)
 
+    async def aio_recvall(self, size: int) -> bytes:
+        """ receive a payload """
+        return await self.reader.readexactly(size)
+
     async def aio_recv_prefixed(self, endianity='>') -> bytes:
         """ receive a data block prefixed with a u32 length field """
-        size = await self._reader.readexactly(4)
+        size = await self.aio_recvall(4)
         size = struct.unpack(endianity + 'L', size)[0]
-        return await self._reader.readexactly(size)
+        return await self.aio_recvall(size)
 
     def send_prefixed(self, data: bytes) -> None:
         """ send a data block prefixed with a u32 length field """
@@ -168,22 +174,25 @@ class LockdownServiceConnection:
     def send_plist(self, d, endianity='>', fmt=plistlib.FMT_XML) -> None:
         return self.sendall(build_plist(d, endianity, fmt))
 
-    async def aio_send_plist(self, d, endianity='>', fmt=plistlib.FMT_XML) -> None:
-        self._writer.write(build_plist(d, endianity, fmt))
-        await self._writer.drain()
+    async def aio_sendall(self, payload: bytes) -> None:
+        self.writer.write(payload)
+        await self.writer.drain()
+
+    async def aio_send_plist(self, d: Mapping, endianity: str = '>', fmt: Enum = plistlib.FMT_XML) -> None:
+        await self.aio_sendall(build_plist(d, endianity, fmt))
 
     def ssl_start(self, certfile, keyfile=None) -> None:
         self.socket = create_context(certfile, keyfile=keyfile).wrap_socket(self.socket)
 
     async def aio_ssl_start(self, certfile, keyfile=None) -> None:
-        self._reader, self._writer = await asyncio.open_connection(
+        self.reader, self.writer = await asyncio.open_connection(
             sock=self.socket,
             ssl=create_context(certfile, keyfile=keyfile),
             server_hostname=''
         )
 
     async def aio_start(self) -> None:
-        self._reader, self._writer = await asyncio.open_connection(sock=self.socket)
+        self.reader, self.writer = await asyncio.open_connection(sock=self.socket)
 
     def shell(self) -> None:
         IPython.embed(
@@ -191,3 +200,15 @@ class LockdownServiceConnection:
             user_ns={
                 'client': self,
             })
+
+    def read(self, size: int) -> bytes:
+        result = self.recvall(size)
+        self._offset += size
+        return result
+
+    def write(self, data: bytes) -> None:
+        self.sendall(data)
+        self._offset += len(data)
+
+    def tell(self) -> int:
+        return self._offset
