@@ -1,6 +1,9 @@
+import copy
 import io
+import os
 import plistlib
 import typing
+import uuid
 from functools import partial
 from pprint import pprint
 from queue import Empty, Queue
@@ -12,8 +15,8 @@ from construct import Adapter, Const, Default, GreedyBytes, GreedyRange, Int16ul
 from pygments import formatters, highlight, lexers
 
 from pymobiledevice3.exceptions import DvtException, UnrecognizedSelectorError
-from pymobiledevice3.lockdown import LockdownClient
-from pymobiledevice3.services.base_service import BaseService
+from pymobiledevice3.lockdown_service_provider import LockdownServiceProvider
+from pymobiledevice3.services.lockdown_service import LockdownService
 
 SHELL_USAGE = '''
 # This shell allows you to send messages to the DVTSecureSocketProxy and receive answers easily.
@@ -114,6 +117,10 @@ class NSNull:
 
 
 class NSError:
+    @staticmethod
+    def encode_archive(archive_obj):
+        return archiver.archive(archive_obj)
+
     @staticmethod
     def decode_archive(archive_obj):
         user_info = archive_obj.decode('NSUserInfo')
@@ -122,6 +129,106 @@ class NSError:
         raise DvtException(archive_obj.decode('NSUserInfo'))
 
 
+class NSUUID(uuid.UUID):
+    @staticmethod
+    def uuid4():
+        """Generate a random UUID."""
+        return NSUUID(bytes=os.urandom(16))
+
+    def encode_archive(self, archive_obj: archiver.ArchivingObject):
+        archive_obj.encode('NS.uuidbytes', self.bytes)
+
+    @staticmethod
+    def decode_archive(archive_obj: archiver.ArchivedObject):
+        return NSUUID(bytes=archive_obj.decode('NS.uuidbytes'))
+
+
+class NSURL:
+    def __init__(self, base, relative):
+        self.base = base
+        self.relative = relative
+
+    def encode_archive(self, archive_obj: archiver.ArchivingObject):
+        archive_obj.encode('NS.base', self.base)
+        archive_obj.encode('NS.relative', self.relative)
+
+    @staticmethod
+    def decode_archive(archive_obj: archiver.ArchivedObject):
+        return NSURL(archive_obj.decode('NS.base'), archive_obj.decode('NS.relative'))
+
+
+class NSValue:
+    @staticmethod
+    def decode_archive(archive_obj: archiver.ArchivedObject):
+        return archive_obj.decode('NS.rectval')
+
+
+class NSMutableData:
+    @staticmethod
+    def decode_archive(archive_obj: archiver.ArchivedObject):
+        return archive_obj.decode('NS.data')
+
+
+class NSMutableString:
+    @staticmethod
+    def decode_archive(archive_obj: archiver.ArchivedObject):
+        return archive_obj.decode('NS.string')
+
+
+class XCTestConfiguration:
+    _default = {
+        # 'testBundleURL': UID(3),
+        # 'sessionIdentifier': UID(8), # UUID
+        'aggregateStatisticsBeforeCrash': {
+            'XCSuiteRecordsKey': {}
+        },
+        'automationFrameworkPath': '/Developer/Library/PrivateFrameworks/XCTAutomationSupport.framework',
+        'baselineFileRelativePath': None,
+        'baselineFileURL': None,
+        'defaultTestExecutionTimeAllowance': None,
+        'disablePerformanceMetrics': False,
+        'emitOSLogs': False,
+        'formatVersion': plistlib.UID(2),  # store in UID
+        'gatherLocalizableStringsData': False,
+        'initializeForUITesting': True,
+        'maximumTestExecutionTimeAllowance': None,
+        'productModuleName': 'WebDriverAgentRunner',  # set to other value is also OK
+        'randomExecutionOrderingSeed': None,
+        'reportActivities': True,
+        'reportResultsToIDE': True,
+        'systemAttachmentLifetime': 2,
+        'targetApplicationArguments': [],  # maybe useless
+        'targetApplicationBundleID': None,
+        'targetApplicationEnvironment': None,
+        'targetApplicationPath': '/whatever-it-does-not-matter/but-should-not-be-empty',
+        'testApplicationDependencies': {},
+        'testApplicationUserOverrides': None,
+        'testBundleRelativePath': None,
+        'testExecutionOrdering': 0,
+        'testTimeoutsEnabled': False,
+        'testsDrivenByIDE': False,
+        'testsMustRunOnMainThread': True,
+        'testsToRun': None,
+        'testsToSkip': None,
+        'treatMissingBaselinesAsFailures': False,
+        'userAttachmentLifetime': 1
+    }
+
+    def __init__(self, kv: dict):
+        assert 'testBundleURL' in kv
+        assert 'sessionIdentifier' in kv
+        self._config = copy.deepcopy(self._default)
+        self._config.update(kv)
+
+    def encode_archive(self, archive_obj: archiver.ArchivingObject):
+        for k, v in self._config.items():
+            archive_obj.encode(k, v)
+
+    @staticmethod
+    def decode_archive(archive_obj: archiver.ArchivedObject):
+        return archive_obj.object
+
+
 archiver.update_class_map({'DTSysmonTapMessage': DTTapMessage,
                            'DTTapHeartbeatMessage': DTTapMessage,
                            'DTTapStatusMessage': DTTapMessage,
@@ -129,22 +236,36 @@ archiver.update_class_map({'DTSysmonTapMessage': DTTapMessage,
                            'DTActivityTraceTapMessage': DTTapMessage,
                            'DTTapMessage': DTTapMessage,
                            'NSNull': NSNull,
-                           'NSError': NSError})
+                           'NSError': NSError,
+                           'NSUUID': NSUUID,
+                           'NSURL': NSURL,
+                           'NSValue': NSValue,
+                           'NSMutableData': NSMutableData,
+                           'NSMutableString': NSMutableString,
+                           'XCTestConfiguration': XCTestConfiguration})
+
+archiver.Archive.inline_types = list(set(archiver.Archive.inline_types + [bytes]))
 
 
 class Channel(int):
     @classmethod
-    def create(cls, value: int, service):
+    def create(cls, value: int, service: 'RemoteServer'):
         channel = cls(value)
         channel._service = service
         return channel
 
+    def receive_key_value(self):
+        return self._service.recv_plist(self)
+
     def receive_plist(self):
         return self._service.recv_plist(self)[0]
 
     def receive_message(self):
         return self._service.recv_message(self)[0]
 
+    def send_message(self, selector: str, args: MessageAux = None, expects_reply: bool = True):
+        self._service.send_message(self, selector, args, expects_reply=expects_reply)
+
     @staticmethod
     def _sanitize_name(name: str):
         """
@@ -187,7 +308,7 @@ class ChannelFragmenter:
                 self._stream_packet_data = b''
 
 
-class RemoteServer(BaseService):
+class RemoteServer(LockdownService):
     """
     Wrapper to Apple's RemoteServer.
     This server exports several ObjC objects allowing calling their respective selectors.
@@ -231,13 +352,14 @@ class RemoteServer(BaseService):
         }
     }
     ```
-    """
+    """  # noqa: E501
     BROADCAST_CHANNEL = 0
     INSTRUMENTS_MESSAGE_TYPE = 2
     EXPECTS_REPLY_MASK = 0x1000
 
-    def __init__(self, lockdown: LockdownClient, service_name, remove_ssl_context=True):
-        super().__init__(lockdown, service_name, is_developer_service=True)
+    def __init__(self, lockdown: LockdownServiceProvider, service_name, remove_ssl_context: bool = True,
+                 is_developer_service: bool = True):
+        super().__init__(lockdown, service_name, is_developer_service=is_developer_service)
 
         if remove_ssl_context and hasattr(self.service.socket, '_sslobj'):
             self.service.socket._sslobj = None
@@ -270,7 +392,8 @@ class RemoteServer(BaseService):
         self.supported_identifiers = aux[0].value
 
     def make_channel(self, identifier) -> Channel:
-        assert identifier in self.supported_identifiers
+        # NOTE: There is also identifier not in self.supported_identifiers
+        # assert identifier in self.supported_identifiers
         if identifier in self.channel_cache:
             return self.channel_cache[identifier]
 
@@ -369,6 +492,19 @@ class RemoteServer(BaseService):
         self.perform_handshake()
         return self
 
+    def close(self):
+        aux = MessageAux()
+        codes = [code for code in self.channel_messages.keys() if code > 0]
+        if codes:
+            for code in codes:
+                aux.append_int(code)
+            try:
+                self.send_message(self.BROADCAST_CHANNEL, '_channelCanceled:', aux, expects_reply=False)
+            except OSError:
+                # ignore: OSError: [Errno 9] Bad file descriptor
+                pass
+        super().close()
+
 
 class Tap:
     def __init__(self, dvt, channel_name: str, config: typing.Mapping):
@@ -387,9 +523,8 @@ class Tap:
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        self.channel.stop(expects_reply=False)
+        self.channel.clear(expects_reply=False)
 
     def __iter__(self):
         while True:
-            for result in self.channel.receive_plist():
-                yield result
+            yield from self.channel.receive_plist()
