@@ -7,14 +7,6 @@ def test_get_apps(lockdown):
         assert len(apps) > 1
 
 
-def test_get_user_apps(lockdown):
-    with InstallationProxyService(lockdown=lockdown) as installation_proxy:
-        app_types = set(
-            [app['ApplicationType'] for app in installation_proxy.get_apps(application_type='User').values()])
-        assert len(app_types) == 1
-        assert app_types.pop() == 'User'
-
-
 def test_get_system_apps(lockdown):
     with InstallationProxyService(lockdown=lockdown) as installation_proxy:
         app_types = set(
