@@ -8,6 +8,6 @@ def test_get_icon_png_data(lockdown):
     Test that getting icon's data returns a valid PNG.
     :param pymobiledevice3.lockdown.LockdownClient lockdown: Lockdown client.
     """
-    springboard = SpringBoardServicesService(lockdown)
-    icon_data = springboard.get_icon_pngdata("com.apple.weather")
-    assert icon_data.startswith(PNG_HEADER)
+    with SpringBoardServicesService(lockdown) as springboard:
+        icon_data = springboard.get_icon_pngdata("com.apple.weather")
+        assert icon_data.startswith(PNG_HEADER)
